#ifndef ServiceTestUtenProcSAN_H_
#define ServiceTestUtenProcSAN_H_

#include "Cpp/BaseClasses/EmptyGroup.h"
#include "Cpp/BaseClasses/PreselectGroup.h"
#include "Cpp/BaseClasses/PostselectGroup.h"
#include "Cpp/BaseClasses/state/StructStateVariable.h"
#include "Cpp/BaseClasses/state/ArrayStateVariable.h"
#include "Cpp/BaseClasses/SAN/SANModel.h" 
#include "Cpp/BaseClasses/SAN/Place.h"
#include "Cpp/BaseClasses/SAN/ExtendedPlace.h"
extern double muMan;
extern double lambdaProc;
extern int numProc;
extern int numFailed;
extern UserDistributions* TheDistribution;

void MemoryError();

#ifndef _proc_header_
#define _proc_header_

typedef int proc_state;
class proc: public ArrayStateVariable<ExtendedPlace<int> > {
  public:
  proc(char* name):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,numProc) {
    char varname[10];
    for (int i=0;i<numProc;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname));
    }
  }

  proc(char* name, int & initialValue):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,numProc) {
    char varname[10];
    for (int i=0;i<numProc;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname, initialValue));
    }
  }

  proc(char* name, int * initialValue):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,numProc) {
    char varname[10];
    for (int i=0;i<numProc;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname, initialValue[i]));
    }
  }
  ~proc() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif
#ifndef _repl_header_
#define _repl_header_

typedef int repl_state[4];
class repl: public ArrayStateVariable<ExtendedPlace<int> > {
  public:
  repl(char* name):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,4) {
    char varname[10];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname));
    }
  }

  repl(char* name, int & initialValue):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,4) {
    char varname[10];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname, initialValue));
    }
  }

  repl(char* name, int * initialValue):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,4) {
    char varname[10];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname, initialValue[i]));
    }
  }
  ~repl() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif

/*********************************************************************
               ServiceTestUtenProcSAN Submodel Definition                   
*********************************************************************/

class ServiceTestUtenProcSAN:public SANModel{
public:

class ProcessorFailActivity:public Activity {
public:

  Place* procFailed;
  short* procFailed_Mobius_Mark;
  proc* Processors;

  double* TheDistributionParameters;
  ProcessorFailActivity();
  ~ProcessorFailActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // ProcessorFailActivityActivity

class ProcessorRepairActivity:public Activity {
public:

  Place* procFailed;
  short* procFailed_Mobius_Mark;
  proc* Processors;

  double* TheDistributionParameters;
  ProcessorRepairActivity();
  ~ProcessorRepairActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // ProcessorRepairActivityActivity

  //List of user-specified place names
  Place* procFailed;
  proc* Processors;

  // Create instances of all actvities
  ProcessorFailActivity ProcessorFail;
  ProcessorRepairActivity ProcessorRepair;
  //Create instances of all groups 
  EmptyGroup ImmediateGroup;

  ServiceTestUtenProcSAN();
  ~ServiceTestUtenProcSAN();
  void CustomInitialization();

  void assignPlacesToActivitiesInst();
  void assignPlacesToActivitiesTimed();
}; // end ServiceTestUtenProcSAN

#endif // ServiceTestUtenProcSAN_H_
