#ifndef serviceNYSAN_H_
#define serviceNYSAN_H_

#include "Cpp/BaseClasses/EmptyGroup.h"
#include "Cpp/BaseClasses/PreselectGroup.h"
#include "Cpp/BaseClasses/PostselectGroup.h"
#include "Cpp/BaseClasses/state/StructStateVariable.h"
#include "Cpp/BaseClasses/state/ArrayStateVariable.h"
#include "Cpp/BaseClasses/SAN/SANModel.h" 
#include "Cpp/BaseClasses/SAN/Place.h"
#include "Cpp/BaseClasses/SAN/ExtendedPlace.h"
extern double muMan;
extern double lambdaProc;
extern double muAut;
extern double lambdaRepl;
extern int replReq;
extern int numProc;
extern int initRepl;
extern int view_card;
extern double lambdaView;
extern UserDistributions* TheDistribution;

void MemoryError();

#ifndef _proc_header_
#define _proc_header_

typedef int proc_state;
class proc: public ArrayStateVariable<ExtendedPlace<int> > {
  public:
  proc(char* name):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,numProc) {
    char varname[10];
    for (int i=0;i<numProc;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname));
    }
  }

  proc(char* name, int & initialValue):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,numProc) {
    char varname[10];
    for (int i=0;i<numProc;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname, initialValue));
    }
  }

  proc(char* name, int * initialValue):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,numProc) {
    char varname[10];
    for (int i=0;i<numProc;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname, initialValue[i]));
    }
  }
  ~proc() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif
#ifndef _repl_header_
#define _repl_header_

typedef int repl_state[4];
class repl: public ArrayStateVariable<ExtendedPlace<int> > {
  public:
  repl(char* name):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,4) {
    char varname[10];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname));
    }
  }

  repl(char* name, int & initialValue):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,4) {
    char varname[10];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname, initialValue));
    }
  }

  repl(char* name, int * initialValue):
    ArrayStateVariable<ExtendedPlace<int> >(name,Int ,4) {
    char varname[10];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new ExtendedPlace<int>(varname, initialValue[i]));
    }
  }
  ~repl() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif

/*********************************************************************
               serviceNYSAN Submodel Definition                   
*********************************************************************/

class serviceNYSAN:public SANModel{
public:

class replicaFailureActivity:public Activity {
public:

  repl* Replica;
  Place* replicaFailed;
  short* replicaFailed_Mobius_Mark;
  Place* replicasFailed;
  short* replicasFailed_Mobius_Mark;

  double* TheDistributionParameters;
  replicaFailureActivity();
  ~replicaFailureActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // replicaFailureActivityActivity

class replicaCreationActivity:public Activity {
public:

  Place* replicaFailed;
  short* replicaFailed_Mobius_Mark;
  Place* procFailed;
  short* procFailed_Mobius_Mark;
  repl* Replica;
  Place* replicasFailed;
  short* replicasFailed_Mobius_Mark;
  Place* Processors;
  short* Processors_Mobius_Mark;

  double* TheDistributionParameters;
  replicaCreationActivity();
  ~replicaCreationActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // replicaCreationActivityActivity

  //List of user-specified place names
  Place* replicaFailed;
  Place* procFailed;
  repl* Replica;

  // Create instances of all actvities
  replicaFailureActivity replicaFailure;
  replicaCreationActivity replicaCreation;
  //Create instances of all groups 
  EmptyGroup ImmediateGroup;

  serviceNYSAN();
  ~serviceNYSAN();
  void CustomInitialization();

  void assignPlacesToActivitiesInst();
  void assignPlacesToActivitiesTimed();
}; // end serviceNYSAN

#endif // serviceNYSAN_H_
