
#include "Study/HWstudy/HWstudyRangeStudy.h"

//******************************************************
//Global Variables
//******************************************************
double lambdaProc;
double muMan;
int numFailed;
int numProc;
int numServices;

//********************************************************
//HWstudyRangeStudy Constructor
//********************************************************
HWstudyRangeStudy::HWstudyRangeStudy() {

  // define arrays of global variable names and types
  NumGVs = 5;
  NumExps = 1;

  GVNames = new char*[NumGVs];
  GVTypes = new char*[NumGVs];
  GVNames[0]=strdup("lambdaProc");
  GVTypes[0]=strdup("double");
  GVNames[1]=strdup("muMan");
  GVTypes[1]=strdup("double");
  GVNames[2]=strdup("numFailed");
  GVTypes[2]=strdup("int");
  GVNames[3]=strdup("numProc");
  GVTypes[3]=strdup("int");
  GVNames[4]=strdup("numServices");
  GVTypes[4]=strdup("int");

  // create the arrays to store the values of each gv
  lambdaProcValues = new double[NumExps];
  muManValues = new double[NumExps];
  numFailedValues = new int[NumExps];
  numProcValues = new int[NumExps];
  numServicesValues = new int[NumExps];

  // call methods to assign values to each gv
  SetValues_lambdaProc();
  SetValues_muMan();
  SetValues_numFailed();
  SetValues_numProc();
  SetValues_numServices();
  SetDefaultMobiusRoot(MOBIUSROOT);
}


//******************************************************
//HWstudyRangeStudy Destructor
//******************************************************
HWstudyRangeStudy::~HWstudyRangeStudy() {
  delete [] lambdaProcValues;
  delete [] muManValues;
  delete [] numFailedValues;
  delete [] numProcValues;
  delete [] numServicesValues;
  delete ThePVModel;
}


//******************************************************
// set values for lambdaProc
//******************************************************
void HWstudyRangeStudy::SetValues_lambdaProc() {
  lambdaProcValues[0] = 0.01;
}


//******************************************************
// set values for muMan
//******************************************************
void HWstudyRangeStudy::SetValues_muMan() {
  muManValues[0] = 0.2;
}


//******************************************************
// set values for numFailed
//******************************************************
void HWstudyRangeStudy::SetValues_numFailed() {
  numFailedValues[0] = 0;
}


//******************************************************
// set values for numProc
//******************************************************
void HWstudyRangeStudy::SetValues_numProc() {
  numProcValues[0] = 4;
}


//******************************************************
// set values for numServices
//******************************************************
void HWstudyRangeStudy::SetValues_numServices() {
  numServicesValues[0] = 3;
}



//******************************************************
//print values of gv (for debugging)
//******************************************************
void HWstudyRangeStudy::PrintGlobalValues(int expNum) {
  if (NumGVs == 0) {
    cout<<"There are no global variables."<<endl;
    return;
  }

  SetGVs(expNum);

  cout<<"The Global Variable values for experiment "<<
    GetExpName(expNum)<<" are:"<<endl;
  cout << "lambdaProc\tdouble\t" << lambdaProc << endl;
  cout << "muMan\tdouble\t" << muMan << endl;
  cout << "numFailed\tint\t" << numFailed << endl;
  cout << "numProc\tint\t" << numProc << endl;
  cout << "numServices\tint\t" << numServices << endl;
}


//******************************************************
//retrieve the value of a global variable
//******************************************************
void *HWstudyRangeStudy::GetGVValue(char *TheGVName) {
  if (strcmp("lambdaProc", TheGVName) == 0)
    return &lambdaProc;
  else if (strcmp("muMan", TheGVName) == 0)
    return &muMan;
  else if (strcmp("numFailed", TheGVName) == 0)
    return &numFailed;
  else if (strcmp("numProc", TheGVName) == 0)
    return &numProc;
  else if (strcmp("numServices", TheGVName) == 0)
    return &numServices;
  else 
    cerr<<"!! HWstudyRangeStudy::GetGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
  return NULL;
}


//******************************************************
//override the value of a global variable
//******************************************************
void HWstudyRangeStudy::OverrideGVValue(char *TheGVName,void *TheGVValue) {
  if (strcmp("lambdaProc", TheGVName) == 0)
    lambdaProc = *(double *)TheGVValue;
  else if (strcmp("muMan", TheGVName) == 0)
    muMan = *(double *)TheGVValue;
  else if (strcmp("numFailed", TheGVName) == 0)
    numFailed = *(int *)TheGVValue;
  else if (strcmp("numProc", TheGVName) == 0)
    numProc = *(int *)TheGVValue;
  else if (strcmp("numServices", TheGVName) == 0)
    numServices = *(int *)TheGVValue;
  else 
    cerr<<"!! HWstudyRangeStudy::OverrideGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
}


//******************************************************
//set the value of all global variables to the given exp
//******************************************************
void HWstudyRangeStudy::SetGVs(int expNum) {
  lambdaProc = lambdaProcValues[expNum];
  muMan = muManValues[expNum];
  numFailed = numFailedValues[expNum];
  numProc = numProcValues[expNum];
  numServices = numServicesValues[expNum];
}


//******************************************************
//static class method called by solvers to create study 
//(and thus create all of the model)
//******************************************************
BaseStudyClass* GlobalStudyPtr = NULL;
BaseStudyClass * GenerateStudy() {
  if (GlobalStudyPtr == NULL)
    GlobalStudyPtr = new HWstudyRangeStudy();
  return GlobalStudyPtr;
}

void DestructStudy() {
  delete GlobalStudyPtr;
  GlobalStudyPtr = NULL;
}
//******************************************************
//get and create the PVModel
//******************************************************
PVModel* HWstudyRangeStudy::GetPVModel(bool expandTimeArrays) {
  if (ThePVModel!=NULL)
    delete ThePVModel;
  // create the PV model
  ThePVModel=new HWavailPVModel(expandTimeArrays);
  return ThePVModel;
}


